package org.insa.graphs.algorithm.shortestpath;

import java.util.ArrayList;
import java.util.Collections;
import org.insa.graphs.model.Node;
import org.insa.graphs.model.Arc;
import org.insa.graphs.model.Path;
import org.insa.graphs.algorithm.AbstractSolution;
import org.insa.graphs.algorithm.utils.BinaryHeap;

public class DijkstraAlgorithm extends ShortestPathAlgorithm {

    public ArrayList<Label> Init(int tailleGraphe, Node destination)
    {

        Label currentLabel;
        ArrayList<Label> labelSommets = new ArrayList<Label>();
        for(int i = 0; i< tailleGraphe; i++ )
        {
            currentLabel = new Label(data.getGraph().get(i));
            labelSommets.add(currentLabel);
        }
        return labelSommets;

    }




    public DijkstraAlgorithm(ShortestPathData data) {
        super(data);
    }

    @Override
    protected ShortestPathSolution doRun() {
        final ShortestPathData data = getInputData();

        int tailleGraphe = data.getGraph().size();
        int index = 0;


        ArrayList<Label> labelSommets = new ArrayList<Label>();

        BinaryHeap<Label> leTas = new BinaryHeap<Label>();

        //Initialisation ----------------------------------------------------

        labelSommets  = Init(tailleGraphe, data.getDestination());
        Label currentLabel;


        currentLabel = labelSommets.get(data.getOrigin().getId());
        currentLabel.setNouveauChemin(null, 0);
        currentLabel.setMarqueTrue();
        leTas.insert(currentLabel);


        Node currentNode = data.getOrigin() ;
        double currentCost;
        double newCost;
        Label newLabel;
        notifyOriginProcessed(currentNode);

        //Boucle principale ----------------------------------------------------


        while (!leTas.isEmpty() && !currentNode.equals(data.getDestination()))
        {

            currentLabel = leTas.deleteMin();
            currentNode = currentLabel.getSommetCourant();
            index = currentNode.getId();
            currentLabel.setMarqueTrue();


            notifyNodeMarked(currentNode);

            
            for(Arc arc : currentNode.getSuccessors())
            {
                if(data.isAllowed(arc))
                {
                    newLabel = labelSommets.get(arc.getDestination().getId());
                    if(!newLabel.getMarque())
                    {
                        currentCost = newLabel.getCost();
                        newCost = data.getCost(arc) + currentLabel.getCost();
                        if(currentCost == -1.0)
                        {
                            newLabel.setNouveauChemin(arc, newCost);
                            leTas.insert(newLabel);
                            notifyNodeReached(newLabel.getSommetCourant());
                        }
                        else if(newCost < currentCost)
                        {
                            leTas.remove(newLabel);
                            newLabel.setNouveauChemin(arc, newCost);
                            leTas.insert(newLabel);
                        }
                    }
                }
                
            }
        }


        //Retour chemin obtenu ----------------------------------------------------

        Node finalNode = data.getDestination();
        index = 0;
        ArrayList<Arc> arcListe = new ArrayList<Arc>() ;

        if(labelSommets.get(data.getDestination().getId()).getCost() == -1)
        {
            return new ShortestPathSolution(data, AbstractSolution.Status.INFEASIBLE);
        }

        while(!finalNode.equals(data.getOrigin())  && index < tailleGraphe)
        {
            arcListe.add(labelSommets.get(finalNode.getId()).pere);
            finalNode = arcListe.get(arcListe.size()-1).getOrigin();
            index++;
        }
        if(index != tailleGraphe)
        {
            Collections.reverse(arcListe);
            notifyDestinationReached(data.getDestination());
            return new ShortestPathSolution(data, AbstractSolution.Status.FEASIBLE, new Path(data.getGraph(),arcListe ));
        }

        return new ShortestPathSolution(data, AbstractSolution.Status.INFEASIBLE);

    }


}
