package org.insa.graphs.algorithm.shortestpath;

import org.insa.graphs.model.Arc;
import org.insa.graphs.model.Node;

public class Label implements Comparable<Label>{

    /* ATTRIBUTS */

    /* Sommet associé à ce label (sommet ou numéro de sommet) */
    protected Node sommetCourant ;

    /* Booléen vrai lorque le coût min de ce sommet est définitivement connu par l'algorithme */
    protected boolean marque ;

    /* Valeur courante du plus court chemin depuis l'origine vers le sommet */
    protected double coutRealise ;

    /* correspond au sommet précédent sur le chemin correspondant au plus court chemin courant */
    protected Arc pere ;

    /* CONSTRUCTEUR */

    public Label(Node sommetCourant) {
        this.sommetCourant = sommetCourant ;
        this.marque = false ;
        this.coutRealise = -1.0 ; //infini
        this.pere = null ; //sommet inexistant
    }

    /* METHODES */

    /* Getter pour sommetCourant */
    public Node getSommetCourant() {
        return this.sommetCourant ;
    }

    /* Getter pour marque */
    public boolean getMarque() {
        return this.marque ;
    }

    /* Getter pour coutRealise */
    public double getCoutRealise() {
        return this.coutRealise ;
    }

    /* Getter pour cout */
    public double getCost() {
        return this.coutRealise ;
    }


    public double getEstimation() {
        return 0 ;
    }

    /* Getter pour pere */
    public Arc getPere() {
        return this.pere ;
    }

    /* Setter Pour Marque */
    public void setMarqueTrue() {
        this.marque = true;
    }

    /* Setter Pour Cout et Pere */
    public void setNouveauChemin(Arc arcPere, double coutRealise){
        this.pere = arcPere;
        this.coutRealise = coutRealise;
    }

    public double getTotalCost() {
        return this.coutRealise; 
    }


/**
     * Compare the ID of this node with the ID of the given node.
     * 
     * @param other Node to compare this node with.
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    @Override
    public int compareTo(Label other) {

        if (this.getTotalCost() < other.getTotalCost())
        {
            return -1;
        }
        else if (this.getTotalCost() > other.getTotalCost())
        {
            return 1;
        }
        return 0;
    }

}


