package org.insa.graphs.algorithm.utils;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.util.ArrayList;

import org.junit.Test;
import org.junit.BeforeClass;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;

import org.insa.graphs.model.Graph;
import org.insa.graphs.model.Path;
import org.insa.graphs.model.io.BinaryGraphReader;
import org.insa.graphs.model.io.BinaryPathReader;

import org.insa.graphs.algorithm.ArcInspector;
import org.insa.graphs.algorithm.ArcInspectorFactory;
import org.insa.graphs.algorithm.shortestpath.AStarAlgorithm;
import org.insa.graphs.algorithm.shortestpath.BellmanFordAlgorithm;
import org.insa.graphs.algorithm.shortestpath.DijkstraAlgorithm;
import org.insa.graphs.algorithm.shortestpath.ShortestPathData;

public class PCCTest{
    final static ArrayList<String> mapNames = new ArrayList<>();
    final static ArrayList<String> pathNames = new ArrayList<>();
    final static ArrayList<Graph> graphs = new ArrayList<>();
    final static ArrayList<Path> paths = new ArrayList<>();
    final static ArrayList<ArcInspector> arcInspectors = new ArrayList<>();

    private static ShortestPathData DataSF_INSA, DataSF_HauteGaronne, DataVL_INSA, DataVL_HauteGaronne, DataVT_INSA, DataVT_HauteGaronne;



    @BeforeClass
    public static void initAll() throws Exception 
    {
        final String nomUser = "brunetto";
        // Visit these directory to see the list of available files on Commetud.

        //Récupération des cartes
        mapNames.add("/home/" + nomUser + "/Bureau/commetud/3eme Annee MIC/Graphes-et-Algorithmes/Maps/insa.mapgr");
        mapNames.add("/home/" + nomUser + "/Bureau/commetud/3eme Annee MIC/Graphes-et-Algorithmes/Maps/haute-garonne.mapgr");
        //Recupération des chemins
        pathNames.add("/home/" + nomUser + "/Bureau/commetud/3eme Annee MIC/Graphes-et-Algorithmes/Paths/path_fr31insa_rangueil_r2.path");
        pathNames.add("/home/" + nomUser + "/Bureau/commetud/3eme Annee MIC/Graphes-et-Algorithmes/Paths/path_fr31_insa_aeroport_length.path");

        // Extraction des données des cartes et chemins

        for(int i = 0; i<2; i++)
        {
            graphs.add(new BinaryGraphReader(new DataInputStream(new BufferedInputStream(new FileInputStream(mapNames.get(i))))).read());
            paths.add(new BinaryPathReader(new DataInputStream(new BufferedInputStream(new FileInputStream(pathNames.get(i))))).readPath(graphs.get(i)));
        }

        arcInspectors.add(ArcInspectorFactory.getAllFilters().get(0)) ; //Sans Filtres
        arcInspectors.add(ArcInspectorFactory.getAllFilters().get(1)); //Voiture, longueur
        arcInspectors.add(ArcInspectorFactory.getAllFilters().get(3)); //Voiture, temps

        //Créations des données pour les algos

        DataSF_INSA = new ShortestPathData(graphs.get(0), paths.get(0).getOrigin(), paths.get(0).getDestination(), arcInspectors.get(0));
        DataSF_HauteGaronne = new ShortestPathData(graphs.get(1), paths.get(1).getOrigin(), paths.get(1).getDestination(), arcInspectors.get(0));
        DataVL_INSA = new ShortestPathData(graphs.get(0), paths.get(0).getOrigin(), paths.get(0).getDestination(), arcInspectors.get(1));
        DataVL_HauteGaronne = new ShortestPathData(graphs.get(1), paths.get(1).getOrigin(), paths.get(1).getDestination(), arcInspectors.get(1));
        DataVT_INSA = new ShortestPathData(graphs.get(0), paths.get(0).getOrigin(), paths.get(0).getDestination(), arcInspectors.get(2));
        DataVT_HauteGaronne = new ShortestPathData(graphs.get(1), paths.get(1).getOrigin(), paths.get(1).getDestination(), arcInspectors.get(2));

       
    }

    private static DijkstraAlgorithm dji ;
    private static AStarAlgorithm as ;
    private static BellmanFordAlgorithm bf ;

    /* TEST DIKJSTRA SANS FILTRE */
    @Test
    public void TestDikjstraSF() {

        dji = new DijkstraAlgorithm(DataSF_INSA);
        bf = new BellmanFordAlgorithm(DataSF_INSA);
        assertEquals(bf.run().getPath().getLength(), dji.run().getPath().getLength(), 0.05f);

        dji = new DijkstraAlgorithm(DataSF_HauteGaronne);
        bf = new BellmanFordAlgorithm(DataSF_HauteGaronne);
        assertEquals(bf.run().getPath().getLength(), dji.run().getPath().getLength(), 0.05f);

    }

    /* TEST DIKJSTRA VOITURE LENGTH */
    @Test
    public void TestDikjstraVL() {

        dji = new DijkstraAlgorithm(DataVL_INSA);
        assertFalse(dji.run().isFeasible());


        dji = new DijkstraAlgorithm(DataVL_HauteGaronne);
        bf = new BellmanFordAlgorithm(DataVL_HauteGaronne);
        assertEquals(bf.run().getPath().getLength(), dji.run().getPath().getLength(), 0.05f);

    }

    /* TEST DIKJSTRA VOITURE TIME */
    @Test
    public void TestDikjstraVT() {


        dji = new DijkstraAlgorithm(DataVL_INSA);
        assertFalse(dji.run().isFeasible());


        dji = new DijkstraAlgorithm(DataVT_HauteGaronne);
        bf = new BellmanFordAlgorithm(DataVT_HauteGaronne);
        assertEquals(bf.run().getPath().getMinimumTravelTime(), dji.run().getPath().getMinimumTravelTime(), 0.05f);

    }

    /* TEST A* SANS FILTRE */
    @Test
    public void TestAStarSF() {
        as = new AStarAlgorithm(DataSF_INSA);
        bf = new BellmanFordAlgorithm(DataSF_INSA);
        assertEquals(bf.run().getPath().getLength(), as.run().getPath().getLength(), 0.05f);
        as = new AStarAlgorithm(DataSF_HauteGaronne);
        bf = new BellmanFordAlgorithm(DataSF_HauteGaronne);
        assertEquals(bf.run().getPath().getLength(), as.run().getPath().getLength(), 0.05f);

    }

    /* TEST A* VOITURE LENGTH */
    @Test
    public void TestAStarVL() {

        as = new AStarAlgorithm(DataVL_INSA);
        assertFalse(as.run().isFeasible());

        as = new AStarAlgorithm(DataVL_HauteGaronne);
        bf = new BellmanFordAlgorithm(DataVL_HauteGaronne);
        assertEquals(bf.run().getPath().getLength(), as.run().getPath().getLength(), 0.05f);

    }

    /* TEST A* VOITURE TIME */
    @Test
    public void TestAStarVT() {

        as = new AStarAlgorithm(DataVT_INSA);
        assertFalse(as.run().isFeasible());

        as = new AStarAlgorithm(DataVT_HauteGaronne);
        bf = new BellmanFordAlgorithm(DataVT_HauteGaronne);
        assertEquals(bf.run().getPath().getMinimumTravelTime(), as.run().getPath().getMinimumTravelTime(), 0.05f);

    }

}

